/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdfbet
*
*      SUBROUTINE CDFBET( WHICH, P, Q, X, Y, A, B, STATUS, BOUND )
*               Cumulative Distribution Function
*                         BETa Distribution

Calling sequences :

CDF :
P=distfun_cdfbet(X,Y,A,B)
[P,Q]=distfun_cdfbet(X,Y,A,B)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_cdfbet(char* fname,unsigned long l)
{

	int readFlag;

	int rowsX = 0, colsX = 0;
	int rowsY = 0, colsY = 0;
	int rowsA = 0, colsA = 0;
	int rowsB = 0, colsB = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;

	double * lrX = NULL;
	double * lrY = NULL;
	double * lrA = NULL;
	double * lrB = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;

	int which = 1;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,2);

	// Arg #1 : X
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrX, &rowsX , &colsX );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Y
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsX, colsX, &lrY, &rowsY , &colsY );
	if(readFlag==0)
	{
		return 0;
	}
	// Check that X+Y~1
	readFlag = distfun_CheckXYEqualOne (fname, lrX, lrY, rowsX , colsX);
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : A
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 3, rowsX, colsX, &lrA, &rowsA , &colsA );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #4 : B
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 4, rowsX, colsX, &lrB, &rowsB , &colsB );
	if(readFlag==0)
	{
		return 0;
	}

	// Create LHS : P, Q
	rowsP = rowsX;
	colsP = colsX;
	rowsQ = rowsX;
	colsQ = colsX;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsP , colsP , &lrP );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsQ , colsQ , &lrQ );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill P, Q : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsX*colsX; i++)
	{
		cdfbet_(&which, lrP+i, lrQ+i, lrX+i, lrY+i, lrA+i, lrB+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		if (status==4)
		{
			Scierror(999,_("%s: Internal error: X + Y not equal to 1 \n"),fname);
		}
		else
		{
			distfun_defaultCDFError( fname, status );
		}
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
