/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
* hand written interface
* Interface for cdfbin
*
*     SUBROUTINE CDFBIN ( WHICH, P, Q, S, XN, PR, OMPR, STATUS, BOUND )
*              Cumulative Distribution Function
*                        BINomial distribution

Calling sequences :

CDF:
P=distfun_cdfbin(S,Xn,Pr,Ompr)
[P,Q]=distfun_cdfbin(S,Xn,Pr,Ompr)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_cdfbin(char* fname,unsigned long l)
{

	int readFlag;

	int rowsS = 0, colsS = 0;
	int rowsXn = 0, colsXn = 0;
	int rowsPr = 0, colsPr = 0;
	int rowsOm = 0, colsOm = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;

	double * lrS = NULL;
	double * lrXn = NULL;
	double * lrPr = NULL;
	double * lrOm = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;

	int which = 1;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,2);

	// Arg #1 : S
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrS, &rowsS , &colsS );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Xn
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsS, colsS, &lrXn, &rowsXn , &colsXn );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : Pr, arg #4 : Ompr
	readFlag = distfun_GetMatricesPQ( fname, 3, rowsS, colsS,
		&lrPr, &rowsPr , &colsPr, &lrOm, &rowsOm , &colsOm);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : P, Q
	rowsP = rowsS;
	colsP = colsS;
	rowsQ = rowsS;
	colsQ = colsS;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsP , colsP , &lrP );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsQ , colsQ , &lrQ );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill P, Q : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsS*colsS; i++)
	{
		cdfbin_(&which, lrP+i, lrQ+i, lrS+i, lrXn+i, lrPr+i, lrOm+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		if (status==4)
		{
			Scierror(999,_("%s: Internal error: Pr + Ompr not equal to 1 \n"),fname);
		}
		else
		{
			distfun_defaultCDFError( fname, status );
		}
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
