/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/
/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*      Interface for cdff
*               Cumulative Distribution Function
*              F distribution

Calling sequences :

CDF:
P=distfun_cdff(F,Dfn,Dfd)
[P,Q]=distfun_cdff(F,Dfn,Dfd)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_cdff(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsF = 0, colsF = 0;
	int rowsDfn = 0, colsDfn = 0;
	int rowsDfd = 0, colsDfd = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;

	double * lrF = NULL;
	double * lrDfn = NULL;
	double * lrDfd = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;

	int which = 1;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(3,3);
	CheckLhs(1,2);

	// Arg #1 : F
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrF, &rowsF , &colsF );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Dfn
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsF, colsF, &lrDfn, &rowsDfn , &colsDfn );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : Dfd
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 3,  rowsF, colsF, &lrDfd, &rowsDfd , &colsDfd );
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : P, Q
	rowsP = rowsF;
	colsP = colsF;
	rowsQ = rowsF;
	colsQ = colsF;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsP , colsP , &lrP );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsQ , colsQ , &lrQ );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill P, Q : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsF*colsF; i++)
	{
		cdff_(&which, lrP+i, lrQ+i, lrF+i, lrDfn+i, lrDfd+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		distfun_defaultCDFError( fname, status );
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
