/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
* Copyright (C) 2010 - DIGITEO - Michael Baudin
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdfgam
*  Cumulative Distribution Function
*  F distribution

Calling sequences :

Inverse CDF:
P=distfun_cdfgam(X,Shape,Rate)
[P,Q]=distfun_cdfgam(X,Shape,Rate)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_cdfgam(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsX = 0, colsX = 0;
	int rowsShape = 0, colsShape = 0;
	int rowsRate = 0, colsRate = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;

	double * lrX = NULL;
	double * lrShape = NULL;
	double * lrRate = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;

	int which = 1;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(3,3);
	CheckLhs(1,2);

	// Arg #1 : X
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrX, &rowsX , &colsX );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Shape
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterThanZero( fname, 2, rowsX, colsX, &lrShape, &rowsShape , &colsShape );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : Rate
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterThanZero( fname, 3, rowsX, colsX, &lrRate, &rowsRate , &colsRate );
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : P, Q
	rowsP = rowsX;
	colsP = colsX;
	rowsQ = rowsX;
	colsQ = colsX;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsP , colsP , &lrP );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsQ , colsQ , &lrQ );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill P, Q : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsX*colsX; i++)
	{
		cdfgam_(&which, lrP+i, lrQ+i, lrX+i, lrShape+i, lrRate+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		if ( status==10 )
		{
			Scierror(999,_("%s: Internal error: The gamma routine cannot compute the answer. Usually happens only for X and SHAPE very large (> 1E10 or more).\n"),fname);
		}
		else
		{
			distfun_defaultCDFError( fname, status );
		}
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
