/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdfpoi
*  POIsson distribution

Calling sequences:

CDF :
P=distfun_cdfpoi(S,Lambda)
[P,Q]=distfun_cdfpoi(S,Lambda)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_cdfpoi(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsLambda = 0, colsLambda = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsS = 0, colsS = 0;

	double * lrLambda = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrS = NULL;

	int which = 1;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(2,2);
	CheckLhs(1,2);

	// Arg #1 : S
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrS, &rowsS , &colsS );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Lambda
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsS, colsS, &lrLambda, &rowsLambda , &colsLambda );
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : P, Q
	rowsP = rowsS;
	colsP = colsS;
	rowsQ = rowsS;
	colsQ = colsS;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsP , colsP , &lrP );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsQ , colsQ , &lrQ );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill P, Q : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsS*colsS; i++)
	{
		cdfpoi_(&which, lrP+i, lrQ+i, lrS+i, lrLambda+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		distfun_defaultCDFError( fname, status );
	}

	return 0;
}
/*--------------------------------------------------------------------------*/

