/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/
/*--------------------------------------------------------------------------*/
#include <string.h>

// From distfun
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdft
*  T distribution

Calling sequences :

CDF:
P=distfun_cdft(T,Df)
[P,Q]=distfun_cdft(T,Df)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_cdft(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsT = 0, colsT = 0;
	int rowsDf = 0, colsDf = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;

	double * lrT = NULL;
	double * lrDf = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;

	int which = 1;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(2,2);
	CheckLhs(1,2);

	// Arg #1 : T
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 1, &lrT, &rowsT , &colsT );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Df
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 2, &lrDf, &rowsDf , &colsDf );
	if(readFlag==0)
	{
		return 0;
	}
	readFlag = gwsupport_CheckSize ( fname , 2, rowsT, colsT, rowsDf , colsDf );
	if (readFlag==0)
	{
		return 0;
	}
	// Create LHS : P, Q
	rowsP = rowsT;
	colsP = colsT;
	rowsQ = rowsT;
	colsQ = colsT;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsP , colsP , &lrP );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsQ , colsQ , &lrQ );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill P, Q : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsT*colsT; i++)
	{
		cdft_(&which, lrP+i, lrQ+i, lrT+i, lrDf+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		distfun_defaultCDFError( fname, status );
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
