/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
static void cdfbetErr(int status,double bound);
/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdfbet
*
*      SUBROUTINE CDFBET( WHICH, P, Q, X, Y, A, B, STATUS, BOUND )
*               Cumulative Distribution Function
*                         BETa Distribution

Calling sequences :

Inverse CDF :
X=distfun_invcdfbet(A,B,P,Q)
[X,Y]=distfun_invcdfbet(A,B,P,Q)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_invcdfbet(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsA = 0, colsA = 0;
	int rowsB = 0, colsB = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsX = 0, colsX = 0;
	int rowsY = 0, colsY = 0;

	double * lrA = NULL;
	double * lrB = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrX = NULL;
	double * lrY = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,2);

	// Arg #1 : A
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1,  -1, -1, &lrA, &rowsA , &colsA );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : B
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsA, colsA, &lrB, &rowsB, &colsB );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : P, arg #4 : Q
	readFlag = distfun_GetMatricesPQ( fname, 3,  rowsA, colsA,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : X, Y
	rowsX = rowsA;
	colsX = colsA;
	rowsY = rowsA;
	colsY = colsA;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsX , colsX , &lrX );
	iAllocMatrixOfDouble ( Rhs + 2 , rowsY , colsY , &lrY );
	LhsVar(1) = Rhs+1;
	LhsVar(2) = Rhs+2;
	// Fill X, Y : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsX*colsX; i++)
	{
		cdfbet_(&which, lrP+i, lrQ+i, lrX+i, lrY+i, lrA+i, lrB+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		if (status==4)
		{
			Scierror(999,_("%s: Internal error: X + Y not equal to 1 \n"),fname);
		}
		else
		{
			distfun_defaultCDFError( fname, status );
		}
	}

	return 0;
}
