/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
* hand written interface
* Interface for cdfbin
*
*     SUBROUTINE CDFBIN ( WHICH, P, Q, S, XN, PR, OMPR, STATUS, BOUND )
*              Cumulative Distribution Function
*                        BINomial distribution

Calling sequences :

Inverse CDF:
S=distfun_invcdfbin(Xn,Pr,Ompr,P,Q)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_invcdfbin(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsXn = 0, colsXn = 0;
	int rowsPr = 0, colsPr = 0;
	int rowsOm = 0, colsOm = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsS = 0, colsS = 0;

	double * lrXn = NULL;
	double * lrPr = NULL;
	double * lrOm = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrS = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(5,5);
	CheckLhs(1,1);

	// Arg #1 : Xn
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrXn, &rowsXn , &colsXn );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Pr, arg #3 : Ompr
	readFlag = distfun_GetMatricesPQ( fname, 2,  rowsXn, colsXn,
		&lrPr, &rowsPr , &colsPr, &lrOm, &rowsOm , &colsOm);
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #4 : P, arg #5 : Q
	readFlag = distfun_GetMatricesPQ( fname, 4,  rowsXn, colsXn,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : S
	rowsS = rowsXn;
	colsS = colsXn;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsS , colsS , &lrS );
	LhsVar(1) = Rhs+1;
	// Fill S : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsS*colsS; i++)
	{
		cdfbin_(&which, lrP+i, lrQ+i, lrS+i, lrXn+i, lrPr+i, lrOm+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		if (status==4)
		{
			Scierror(999,_("%s: Internal error: Pr + Ompr not equal to 1 \n"),fname);
		}
		else
		{
			distfun_defaultCDFError( fname, status );
		}
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
