/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/
/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdfchn
*
*      SUBROUTINE CDFCHN( WHICH, P, Q, X, DF, PNONC, STATUS, BOUND ) *
*               Cumulative Distribution Function *
*               Non-central Chi-Square *
*
* Calling sequences :
* 
* Inverse CDF:
* X=distfun_invcdfchn(Df,Pnonc,P,Q);
* 
*/
int sci_distfun_invcdfchn(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsDf = 0, colsDf = 0;
	int rowsPnonc = 0, colsPnonc = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsX = 0, colsX = 0;

	double * lrDf = NULL;
	double * lrPnonc = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrX = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Arg #1 : Df
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrDf, &rowsDf , &colsDf );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Pnonc
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsDf, colsDf, &lrPnonc, &rowsPnonc , &colsPnonc );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : P, arg #4 : Q
	readFlag = distfun_GetMatricesPQ( fname, 3,  rowsDf, colsDf,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : X
	rowsX = rowsDf;
	colsX = colsDf;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsX , colsX , &lrX );
	LhsVar(1) = Rhs+1;
	// Fill S : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsDf*colsDf; i++)
	{
		cdfchn_(&which, lrP+i, lrQ+i, lrX+i, lrDf+i, lrPnonc+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		distfun_defaultCDFError( fname, status );
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
