/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/
/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*      Interface for cdff
*               Cumulative Distribution Function
*              F distribution

Calling sequences :

Inverse CDF:
F=distfun_invcdff(Dfn,Dfd,P,Q);

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_invcdff(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsDfn = 0, colsDfn = 0;
	int rowsDfd = 0, colsDfd = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsF = 0, colsF = 0;

	double * lrDfn = NULL;
	double * lrDfd = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrF = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Arg #1 : Dfn
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrDfn, &rowsDfn , &colsDfn );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Dfd
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsDfn, colsDfn, &lrDfd, &rowsDfd , &colsDfd );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : P, arg #4 : Q
	readFlag = distfun_GetMatricesPQ( fname, 3,  rowsDfn, colsDfn,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : F
	rowsF = rowsDfn;
	colsF = colsDfn;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsF , colsF , &lrF );
	LhsVar(1) = Rhs+1;
	// Fill S : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsDfn*colsDfn; i++)
	{
		cdff_(&which, lrP+i, lrQ+i, lrF+i, lrDfn+i, lrDfd+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		distfun_defaultCDFError( fname, status );
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
