/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
* Copyright (C) 2010 - DIGITEO - Michael Baudin
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
*  hand written interface
*  Interface for cdfgam
*  Cumulative Distribution Function
*  F distribution

Calling sequences :

Inverse CDF:
X=distfun_cdfgam(Shape,Rate,P,Q)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_invcdfgam(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsShape = 0, colsShape = 0;
	int rowsRate = 0, colsRate = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsX = 0, colsX = 0;

	double * lrShape = NULL;
	double * lrRate = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrX = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Arg #1 : Shape
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterThanZero( fname, 1,  -1, -1, &lrShape, &rowsShape , &colsShape );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Rate
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterThanZero( fname, 2, rowsShape, colsShape, &lrRate, &rowsRate , &colsRate );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : P, arg #4 : Q
	readFlag = distfun_GetMatricesPQ( fname, 3,  rowsShape, colsShape,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : X
	rowsX = rowsShape;
	colsX = colsShape;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsX , colsX , &lrX );
	LhsVar(1) = Rhs+1;
	// Fill X : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsShape*colsShape; i++)
	{
		cdfgam_(&which, lrP+i, lrQ+i, lrX+i, lrShape+i, lrRate+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		if ( status==10 )
		{
			Scierror(999,_("%s: Internal error: The gamma routine cannot compute the answer. Usually happens only for X and SHAPE very large (> 1E10 or more).\n"),fname);
		}
		else
		{
			distfun_defaultCDFError( fname, status );
		}
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
