/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*
* hand written interface
* Interface for cdfnor
* Cumulative Distribution Function
* NORmal distribution

Calling sequences:

Inverse CDF :
X=distfun_invcdfnor(Mean,Std,P,Q)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_invcdfnor(char* fname,unsigned long l)
{
	int *piAddr = NULL;

	int readFlag;

	int rowsMean = 0, colsMean = 0;
	int rowsStd = 0, colsStd = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsX = 0, colsX = 0;

	double * lrMean = NULL;
	double * lrStd = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrX = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Arg #1 : Mean
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 1, &lrMean, &rowsMean , &colsMean );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #2 : Std
	// TODO : update to check that Std > 0
	readFlag = distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( fname, 2, rowsMean, colsMean, &lrStd, &rowsStd , &colsStd );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #3 : P, arg #4 : Q
	readFlag = distfun_GetMatricesPQ( fname, 3,  rowsMean, colsMean,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==0)
	{
		return 0;
	}
	// Create LHS : X
	rowsX = rowsMean;
	colsX = colsMean;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsX , colsX , &lrX );
	LhsVar(1) = Rhs+1;
	// Fill X : if OK, status=0. If not OK, status!=0.
	for ( i=0 ; i < rowsMean*colsMean; i++)
	{
		cdfnor_(&which, lrP+i, lrQ+i, lrX+i, lrMean+i, lrStd+i, &status, &bound);

		if (status != 0)
		{
			break;
		}
	}
	if ( status!=0 )
	{
		distfun_defaultCDFError( fname, status );
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
