
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

/**************************************************
Y=distfun_grandexp(m,n,Av) 

generates random variates from the exponential 
distribution with mean Av (real >= 0.0).

***********************************************************************/

int sci_distfun_grandexp(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int readFlag;
	
	int mAv, nAv;
	double * pAv;

	CheckRhs(3,3);
	CheckLhs(0,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname, &rRows, &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Arg #3 : Av
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pAv, &mAv, &nAv);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixGreaterOrEqual (fname, 3, pAv, mAv, nAv, 0.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the argument dAv)
	readFlag = distfun_computeRandgenA (fname, mAv, nAv, pAv, rRows, rCols, genexp);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
