
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

/**************************************************
Y=distfun_grandgam(m,n,shape,rate) 

generates random variates from the gamma distribution with parameters shape (real > 0) and rate (real > 0). 

The density of the gamma is :

shape  (shape-1)   -rate x
rate       x          e          /  gamma(shape)

Related function(s) : gamma, cdfgam.
***********************************************************************/

int sci_distfun_grandgam(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int ma, na;
	int mb, nb;
	double minlog=1.e-37;
	int readFlag;
	double * pa = NULL;
	double * pb = NULL;


	CheckRhs(4,4);
	CheckLhs(1,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname , &rRows , &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #3 : a
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pa, &ma, &na);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that a>0
	readFlag = gwsupport_CheckDoubleMatrixGreaterThan (fname, 3, pa, ma, na, minlog);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #4 : b
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 4, &pb, &mb, &nb);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that b>0
	readFlag = gwsupport_CheckDoubleMatrixGreaterThan (fname, 4, pb, mb, nb, minlog);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the arguments a and b)
	/** WARNING : order (a,b) is changed to (b,a) in parameters for
	compatibility between distfun_grandgam() and distfun_cdfgam()
	**/
	readFlag = distfun_computeRandgenAB (fname, mb, nb, pb, ma, na, pa, rRows, rCols, gengam );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
