
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

// mygengeom --
//
// Generate a random variable from Geometric distribution.
//
// Arguments
// p (input): the probability
// R (output): the outcome
//
// Note
// Encapsulates R=igngeom(p), so that it has the calling sequence 
// required by distfun_computeRandgenA.
// The difference is that p is a (double *) instead of a (double).
double mygengeom (double * p)
{
	double R;
	R= igngeom(*p);
	return R;
}

/**************************************************
Y=distfun_grandgeom(m,n,p) 

generates random variates from the geometric distribution with parameter p : 
number of Bernouilli trials (with probability succes of p) until a succes is met. 
p must be in [pmin,1] (with pmin = 1.3 10^(-307)).

Y contains positive real numbers with integer values, with are the "number of trials to get a success".
***********************************************************************/

int sci_distfun_grandgeom(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int readFlag;
	int ma, na;

	double * pa = NULL;

	CheckRhs(3,3);
	CheckLhs(1,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname , &rRows , &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #3 : p
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pa, &ma, &na);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixInRange (fname, 3, pa, ma , na, 1.3e-307, 1.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the argument df)
	readFlag = distfun_computeRandgenA (fname, ma, na, pa, rRows, rCols, mygengeom);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
