
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "Scierror.h"
#include "sciprint.h"
#include "localization.h"
#include "stack-c.h"

/**************************************************
Y=distfun_grandmn(m,n,Mean,Cov) 

generates n multivariate normal random variates ; 

Mean must be a m x 1 matrix and Cov a m x m symmetric positive 
definite matrix 

Y is then a m x n matrix
***********************************************************************/

int sci_distfun_grandmn(char *fname,unsigned long fname_len)
{
	int ResL,ResC,m2,n2,m1,n1,l1,la,lr,lb;
	int i;
	int nn,un=1,work,mp,parm,ierr;

	CheckRhs(3,3);
	CheckLhs(1,1);

    GetRhsVar(1,MATRIX_OF_INTEGER_DATATYPE, &ResL, &ResC, &l1);
    if ( ResL*ResC != 1)
    {
		Scierror(999,_("%s: Wrong value for first input argument: Must be the number of random simulation.\n"),fname);
		return 0;
    }
	nn= *istk(l1);
	GetRhsVar(2,MATRIX_OF_DOUBLE_DATATYPE, &m1, &n1, &la);
	if ( n1 != 1) 
	{ 
		Scierror(999,_("%s: Wrong type for input argument: Column vector expected.\n"),fname);
		return 0;
	}
	GetRhsVar(3,MATRIX_OF_DOUBLE_DATATYPE, &m2, &n2, &lb);
	if ( m2 != n2 ) 
	{ 
		Scierror(999,_("%s: Wrong type for input argument: Square matrix expected.\n"),fname);
		return 0;
	}
	if ( m2 != m1 ) 
	{ 
		Scierror(999,_("%s: Wrong type for input arguments: Mean and Cov have incompatible dimensions\n"),fname);
		return 0;
	}

	CreateVar(Rhs+1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&nn,&lr);
	CreateVar(Rhs+2,MATRIX_OF_DOUBLE_DATATYPE,&m1,&un,&work);
	mp=m1*(m1+3)/2 + 1;
	CreateVar(Rhs+3,MATRIX_OF_DOUBLE_DATATYPE,&mp,&un,&parm);
	if ( m1 <= 0 )
	{
		Scierror(999,_("%s: Wrong size for input arguments: Mean and Cov are of null size.\n"),fname);
		return 0;
	}
	setgmn(stk(la),stk(lb),&m2,&m1,stk(parm),&ierr);
	if ( ierr == 1)
	{
		SciError(999);
		return 0;
	}
	for ( i=0 ; i < nn ; i++)
	{
		genmn(stk(parm),stk(lr+(m1)*i),stk(work));
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
