
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "Scierror.h"
#include "sciprint.h"
#include "localization.h"
#include "stack-c.h"

/**************************************************
Y=distfun_grandmul(n,nb,P) 

generates n observations from the Multinomial distribution : 

class nb events in m categories (put nb "balls" in m "boxes"). 

P(i) is the probability that an event will be classified into category i. 

P the vector of probabilities is of size m-1 (the probability of category m being 1-sum(P)). 

Y is of size m x n, each column Y(:,j) being an observation from multinomial 
distribution and Y(i,j) the number of events falling in category i (for the j th observation) 

sum(Y(:,j)) = nb.
***********************************************************************/

int sci_distfun_grandmul(char *fname,unsigned long fname_len)
{
	int minrhs = 1,maxrhs = 10,minlhs=1,maxlhs=2;
	int rRows,rCols,m2,n2,m1,n1,l1,la,lr,lb;
	int l_i,nn,ncat;
	double ptot;
	

	CheckRhs(3,3);
	CheckLhs(1,1);

	GetRhsVar(1,MATRIX_OF_INTEGER_DATATYPE, &rRows, &rCols, &l1);
	if ( rRows*rCols != 1)
	{ 
		Scierror(999,_("%s: Wrong value for first input argument: Must be the number of random deviate.\n"),fname);
		return 0;
	}
	nn= *istk(l1);
	GetRhsVar(2,MATRIX_OF_INTEGER_DATATYPE, &m1, &n1, &la);
	if ( m1*n1 != 1) 
	{ 
		Scierror(999,_("%s: Wrong size for input argument: Scalar expected for %s.\n"),fname,"N");
		return 0;
	}
	GetRhsVar(3,MATRIX_OF_DOUBLE_DATATYPE, &m2, &n2, &lb);
	if ( n2 != 1 )
	{
		Scierror(999,_("%s: Wrong size for input argument: Column vector expected.\n"),fname);
		return 0;
	}
	ncat = m2+1;
	CreateVar(Rhs+1,MATRIX_OF_INTEGER_DATATYPE,&ncat,&nn,&lr);
	if ( *istk(la) < 0 )
	{
		Scierror(999,_("%s: N < 0\n"),fname);
		return 0;
	}
	if ( ncat <= 1)
	{
		Scierror(999,_("%s: Ncat <= 1\n"),fname);
		return 0;
	}
	ptot = 0.0;
	for ( l_i= 0 ; l_i < ncat -1 ; l_i++ )
	{
		if ( *stk(lb+l_i) < 0.0 )
		{
			Scierror(999,_("%s: P(%d) < 0\n"),fname,l_i+1);
			return 0;
		}
		if ( *stk(lb+l_i) > 1.0 )
		{
			Scierror(999,_("%s: P(%d) > 1\n"),fname,l_i+1);
			return 0;
		}
		ptot += *stk(lb+l_i);
	}
	if ( ptot > 1.0)
	{
		Scierror(999,_("%s: Sum of P(i) > 1\n"),fname);
		return 0;
	}
	for ( l_i=0 ; l_i < nn ; l_i++)
	{
		genmul(istk(la),stk(lb),&ncat,istk(lr+ncat*l_i));
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
