
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

// mygennbn --
//
// Generate a random variable from Negative Binomial distribution.
//
// Arguments
// p (input): the probability
// R (output): the outcome
//
// Note
// Encapsulates R=ignnbn(p), so that it has the calling sequence 
// required by distfun_computeRandgenAB.
// The difference is that
// * Y is a (double) instead of a (int).
// * N is a (double *) instead of a (int *).
double mygennbn (double * N , double * p)
{
	double R;
	int integerR;
	int integerN;
	
	integerN = (int) (*N);
	integerR = ignnbn(&integerN,p);
	R = (double) integerR;
	return R;
}

/**************************************************
Y=distfun_grandnbn(m,n,N,p) 

generates random variates from the negative binomial distribution 
with parameters N (positive integer or real) and p (real in (0,1)) : 
number of failures occurring before N successes in independant Bernouilli trials 
with probability p of success. 

On output, Y is a double, with integer value.

Related function(s) : cdfnbn.
***********************************************************************/

int sci_distfun_grandnbn(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int ma, na;
	int mb, nb;
	int readFlag;
	double * pa = NULL;
	double * pb = NULL;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname , &rRows , &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #3 : a
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pa, &ma, &na);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that a>=0
	readFlag = gwsupport_CheckDoubleMatrixGreaterOrEqual (fname, 3, pa, ma, na, 0.);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that a has integer value
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue (fname, 3, pa, ma, na);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #4 : b
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 4, &pb, &mb, &nb);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that b in [0,1]
	readFlag = gwsupport_CheckDoubleMatrixInRange (fname, 4, pb, mb, nb, 0., 1.);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the arguments a and b)
	readFlag = distfun_computeRandgenAB (fname, ma, na, pa, mb, nb, pb, rRows, rCols, mygennbn );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
