
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

// mygenpoi --
//
// Generate a random variable from Poisson distribution.
//
// Arguments
// mu (input): the average
// R (output): the outcome
//
// Note
// Encapsulates R=ignpoi(mu), so that it has the calling sequence 
// required by distfun_computeRandgenA.
// The difference is that R is a (double) instead of a (int).
double mygenpoi (double * mu)
{
	int integerR;
	double R;
	integerR= ignpoi(mu);
	R = (double) integerR;
	return R;
}

/**************************************************
Y=distfun_grandpoi(m,n,mu) 

generates random variates from the Poisson distribution 
with mean mu (real >= 0.0). 
Y is a double, with an integer value.

Related function(s) : cdfpoi.
***********************************************************************/

int sci_distfun_grandpoi(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int readFlag;
	
	int mmu, nmu;
	double * pmu;

	CheckRhs(3,3);
	CheckLhs(0,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname, &rRows, &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Arg #3 : mu
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pmu, &mmu, &nmu);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixGreaterThan (fname, 3, pmu, mmu, nmu, 0.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the argument dmu)
	readFlag = distfun_computeRandgenA (fname, mmu, nmu, pmu, rRows, rCols, mygenpoi);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
