
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 
#include "unifrng.h"

// From Scilab:
#include "api_scilab.h"
#include "Scierror.h"
#include "localization.h"
#include "stack-c.h"

/**************************************************
Y=distfun_granduin(m,n,Low,High) 

generates random integers uniformly distributed between Low and High (included). 

High and Low must be integers such that (High-Low+1) < 2,147,483,561.
***********************************************************************/

int sci_distfun_granduin(char *fname,unsigned long fname_len)
{
	int ma, na;
	int mb, nb;
	int rRows,rCols;
	int i;
	int readFlag;
	
	double * pa = NULL;
	double * pb = NULL;

	CheckRhs(4,4);
	CheckLhs(1,1);

	// Get Arg #1 = M, Arg #2 = N
	readFlag = distfun_GetMN ( fname , &rRows , &rCols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #3 : a
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 3, &pa, &ma, &na);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue (fname, 3, pa, ma, na );
	if ( readFlag == 0)
	{ 
		return 0;
	}
    // Arg #4 : b
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 4, &pb, &mb, &nb);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue (fname, 4, pb, mb, nb );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that size(a)==size(b)
	readFlag = gwsupport_CheckSize ( fname, 4, ma, na, mb, nb);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Check that (b-a+1) <= 2147483561
	for ( i=0 ; i < ma*na ; i++)
	{
		if ( (*(pb+i)-*(pa+i)+1) > 2147483561 )
		{
			Scierror(999,_("%s: Wrong value for input argument #%d: ""%s"" expected .\n"),fname,3,"(b-a+1) <= 2147483561");
			return 0;
		}
	}
    // Compute the result (if necessary, expand the arguments a and b)
	readFlag = distfun_computeRandgenAB (fname, ma, na, pa, mb, nb, pb, rRows, rCols, unifrng_generateIntegerInRange );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;
	return 0;
}
