/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfchn.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
 */

#include "cdflib.h"


/* Table of constant values */

static double c_b15 = 0.;
static double c_b17 = .5;
static double c_b19 = 5.;
static double c_b20 = 1e-50;
static double c_b40 = 1e4;

/* Subroutine */ int cdfchn_(int *which, double *p, double *q, 
	double *x, double *df, double *pnonc, int *status, 
	double *bound)
{
	static double fx;
	static long int qhi;
	static double cum, ccum;
	static long int qleft;

	double inf = cdflib_inf();
	double rtol = cdflib_rtol();
	double zero = cdflib_zero();

	/* ********************************************************************** */

	/*      SUBROUTINE CDFCHN( WHICH, P, Q, X, DF, PNONC, STATUS, BOUND ) */
	/*               Cumulative Distribution Function */
	/*               Non-central Chi-Square */


	/*                              Function */


	/*     Calculates any one parameter of the non-central chi-square */
	/*     distribution given values for the others. */


	/*                              Arguments */


	/*     WHICH --> Integer indicating which of the next three argument */
	/*               values is to be calculated from the others. */
	/*               Input range: 1..4 */
	/*               iwhich = 1 : Calculate P and Q from X and DF */
	/*               iwhich = 2 : Calculate X from P,DF and PNONC */
	/*               iwhich = 3 : Calculate DF from P,X and PNONC */
	/*               iwhich = 3 : Calculate PNONC from P,X and DF */
	/*                    INTEGER WHICH */

	/*     P <--> The integral from 0 to X of the non-central chi-square */
	/*            distribution. */
	/*            Input range: [0, 1-1E-16). */
	/*                    DOUBLE PRECISION P */

	/*     Q <--> 1-P. */
	/*            Q is not used by this subroutine and is only included */
	/*            for similarity with other cdf* routines. */
	/*                    DOUBLE PRECISION Q */

	/*     X <--> Upper limit of integration of the non-central */
	/*            chi-square distribution. */
	/*            Input range: [0, +infinity). */
	/*            Search range: [0,inf] */
	/*                    DOUBLE PRECISION X */

	/*     DF <--> Degrees of freedom of the non-central */
	/*             chi-square distribution. */
	/*             Input range: (0, +infinity). */
	/*             Search range: [ zero, inf] */
	/*                    DOUBLE PRECISION DF */

	/*     PNONC <--> Non-centrality parameter of the non-central */
	/*                chi-square distribution. */
	/*                Input range: [0, +infinity). */
	/*                Search range: [0,1E4] */
	/*                    DOUBLE PRECISION PNONC */

	/*     STATUS <-- 0 if calculation completed correctly */
	/*               -I if input parameter number I is out of range */
	/*                1 if answer appears to be lower than lowest */
	/*                  search bound */
	/*                2 if answer appears to be higher than greatest */
	/*                  search bound */
	/*                    INTEGER STATUS */

	/*     BOUND <-- Undefined if STATUS is 0 */

	/*               Bound exceeded by parameter number I if STATUS */
	/*               is negative. */

	/*               Lower search bound if STATUS is 1. */

	/*               Upper search bound if STATUS is 2. */


	/*                              Method */


	/*     Formula  26.4.25   of   Abramowitz   and   Stegun,  Handbook  of */
	/*     Mathematical  Functions (1966) is used to compute the cumulative */
	/*     distribution function. */

	/*     Computation of other parameters involve a seach for a value that */
	/*     produces  the desired  value  of P.   The search relies  on  the */
	/*     monotinicity of P with the other parameter. */


	/*                            WARNING */

	/*     The computation time  required for this  routine is proportional */
	/*     to the noncentrality  parameter  (PNONC).  Very large  values of */
	/*     this parameter can consume immense  computer resources.  This is */
	/*     why the search range is bounded by 10,000. */

	/* ********************************************************************** */
	/*     .. Parameters .. */
	/*     .. */
	/*     .. Scalar Arguments .. */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */

	/*     .. */
	/*     .. Executable Statements .. */

	/*     Check arguments */

	if (! (*which < 1 || *which > 4)) {
		goto L30;
	}
	if (! (*which < 1)) {
		goto L10;
	}
	*bound = 1.;
	goto L20;
L10:
	*bound = 4.;
L20:
	*status = -1;
	return 0;
L30:
	if (*which == 1) {
		goto L70;
	}

	/*     P */

	if (! (*p < 0. || *p > .99999999999999989)) {
		goto L60;
	}
	if (! (*p < 0.)) {
		goto L40;
	}
	*bound = 0.;
	goto L50;
L40:
	*bound = .99999999999999989;
L50:
	*status = -2;
	return 0;
L60:
L70:
	if (*which == 2) {
		goto L90;
	}

	/*     X */

	if (! (*x < 0.)) {
		goto L80;
	}
	*bound = 0.;
	*status = -4;
	return 0;
L80:
L90:
	if (*which == 3) {
		goto L110;
	}

	/*     DF */

	if (! (*df <= 0.)) {
		goto L100;
	}
	*bound = 0.;
	*status = -5;
	return 0;
L100:
L110:
	if (*which == 4) {
		goto L130;
	}

	/*     PNONC */

	if (! (*pnonc < 0.)) {
		goto L120;
	}
	*bound = 0.;
	*status = -6;
	return 0;
L120:

	/*     Calculate ANSWERS */

L130:
	if (1 == *which) {

		/*     Calculating P and Q */

		cumchn_(x, df, pnonc, p, q);
		*status = 0;
	} else if (2 == *which) {

		/*     Calculating X */

		*x = 5.;
		dstinv_(&c_b15, &inf, &c_b17, &c_b17, &c_b19, &c_b20, &rtol);
		*status = 0;
		dinvr_(status, x, &fx, &qleft, &qhi);
L140:
		if (! (*status == 1)) {
			goto L150;
		}
		cumchn_(x, df, pnonc, &cum, &ccum);
		fx = cum - *p;
		dinvr_(status, x, &fx, &qleft, &qhi);
		goto L140;
L150:
		if (! (*status == -1)) {
			goto L180;
		}
		if (! qleft) {
			goto L160;
		}
		*status = 1;
		*bound = 0.;
		goto L170;
L160:
		*status = 2;
		*bound = inf;
L170:
L180:
		;
	} else if (3 == *which) {

		/*     Calculating DF */

		*df = 5.;
		dstinv_(&zero, &inf, &c_b17, &c_b17, &c_b19, &c_b20, &rtol);
		*status = 0;
		dinvr_(status, df, &fx, &qleft, &qhi);
L190:
		if (! (*status == 1)) {
			goto L200;
		}
		cumchn_(x, df, pnonc, &cum, &ccum);
		fx = cum - *p;
		dinvr_(status, df, &fx, &qleft, &qhi);
		goto L190;
L200:
		if (! (*status == -1)) {
			goto L230;
		}
		if (! qleft) {
			goto L210;
		}
		*status = 1;
		*bound = zero;
		goto L220;
L210:
		*status = 2;
		*bound = inf;
L220:
L230:
		;
	} else if (4 == *which) {

		/*     Calculating PNONC */

		*pnonc = 5.;
		dstinv_(&c_b15, &c_b40, &c_b17, &c_b17, &c_b19, &c_b20, &rtol);
		*status = 0;
		dinvr_(status, pnonc, &fx, &qleft, &qhi);
L240:
		if (! (*status == 1)) {
			goto L250;
		}
		cumchn_(x, df, pnonc, &cum, &ccum);
		fx = cum - *p;
		dinvr_(status, pnonc, &fx, &qleft, &qhi);
		goto L240;
L250:
		if (! (*status == -1)) {
			goto L280;
		}
		if (! qleft) {
			goto L260;
		}
		*status = 1;
		*bound = zero;
		goto L270;
L260:
		*status = 2;
		*bound = 1e4;
L270:
L280:
		;
	}
	return 0;
} /* cdfchn_ */

