/* dinvnr.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include "cdflib.h"
#include <math.h>

double dinvnr_(double *p, double *q)
{
	/* System generated locals */
	double ret_val, d__1;

	/* Local variables */
	static int i__;
	static double dx, pp, cum, ccum, xcur;
	static long int qporq;
	static double strtx;

	/* ********************************************************************** */

	/*     DOUBLE PRECISION FUNCTION DINVNR(P,Q) */
	/*     Double precision NoRmal distribution INVerse */


	/*                              Function */


	/*     Returns X  such that CUMNOR(X)  =   P,  i.e., the  integral from - */
	/*     infinity to X of (1/SQRT(2*PI)) EXP(-U*U/2) dU is P */


	/*                              Arguments */


	/*     P --> The probability whose normal deviate is sought. */
	/*                    P is DOUBLE PRECISION */

	/*     Q --> 1-P */
	/*                    P is DOUBLE PRECISION */


	/*                              Method */


	/*     The  rational   function   on  page 95    of Kennedy  and  Gentle, */
	/*     Statistical Computing, Marcel Dekker, NY , 1980 is used as a start */
	/*     value for the Newton method of finding roots. */


	/*                              Note */


	/*     If P or Q .lt. machine EPS returns +/- DINVNR(EPS) */

	/* ********************************************************************** */
	/*     .. Parameters .. */
	/*     .. */
	/*     .. Scalar Arguments .. */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */

	/*     .. */

	/*     .. */
	/*     .. Statement Functions .. */
	/*     .. */
	/*     .. Executable Statements .. */

	/*     FIND MINIMUM OF P AND Q */

	qporq = *p <= *q;
	if (! qporq) {
		pp = *q;
	} else {
		pp = *p;
	}

	/* DIGITEO - M. Baudin - 2010 */
	/* Patched for bug #8032. */
	/* Avoid to enter in the zero-finder algorithm in trivial case. */

	if (pp == .5) {
		ret_val = 0.;
		return ret_val;
	}

	/*     INITIALIZATION STEP */

	strtx = stvaln_(&pp);
	xcur = strtx;

	/*     NEWTON INTERATIONS */

	for (i__ = 1; i__ <= 100; ++i__) {
		cumnor_(&xcur, &cum, &ccum);
		dx = (cum - pp) / (.3989422804014326 * exp(-.5 * xcur * xcur));
		xcur -= dx;
		if ((d__1 = dx / xcur, fabs(d__1)) < 1e-13) {
			goto L40;
		}
		/* L30: */
	}
	ret_val = strtx;

	/*     IF WE GET HERE, NEWTON HAS FAILED */

	if (! qporq) {
		ret_val = -ret_val;
	}
	return ret_val;

	/*     IF WE GET HERE, NEWTON HAS SUCCEDED */

L40:
	ret_val = xcur;
	if (! qporq) {
		ret_val = -ret_val;
	}
	return ret_val;
} /* dinvnr_ */

