/* lnp1m1.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "genrand.h"

/* Copyright (C) Bruno Pincon */

/* This file must be used under the terms of the CeCILL. */
/* This source file is licensed as described in the file COPYING, which */
/* you should have received as part of this distribution.  The terms */
/* are also available at */
/* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt */

double lnp1m1(double *s)
{
	/* System generated locals */
	double ret_val;

	/* Local variables */
	static double s2;


	/*     PURPOSE :  Compute   v = log ( (1 + s)/(1 - s) ) */
	/*        for small s, this is for |s| < SLIM = 0.20 */

	/*     ALGORITHM : */
	/*     1/ if |s| is "very small" we use a truncated */
	/*        taylor dvp (by keeping 3 terms) from : */
	/*                               2        4          6 */
	/*        t = 2 * s * ( 1 + 1/3 s  + 1/5 s  + [ 1/7 s  + ....] ) */
	/*                               2        4 */
	/*        t = 2 * s * ( 1 + 1/3 s  + 1/5 s  + er) */

	/*        The limit E until we use this formula may be simply */
	/*        gotten so that the negliged part er is such that : */
	/*                                    2        4 */
	/*        (#) er <= epsm * ( 1 + 1/3 s  + 1/5 s )   for all |s|<= E */

	/*        As  er  = 1/7 s^6 + 1/9 s^8 + ... */
	/*            er <= 1/7 * s^6 ( 1 + s^2 + s^4 + ...) = 1/7  s^6/(1-s^2) */

	/*        the inequality (#) is forced if : */

	/*        1/7  s^6 / (1-s^2)  <= epsm * ( 1 + 1/3 s^2  + 1/5 s^4 ) */

	/*        s^6 <= 7 epsm * (1 - 2/3 s^2 - 3/15 s^4 - 1/5 s^6) */

	/*        So that E is very near (7 epsm)^(1/6) (approximately 3.032d-3): */

	/*     2/ For larger |s| we used a minimax polynome : */

	/*        yi = s * (2  + d3 s^3 + d5 s^5 .... + d13 s^13 + d15 s^15) */

	/*        This polynome was computed (by some remes algorithm) following */
	/*        (*) the sin(x) example (p 39) of the book : */

	/*         "ELEMENTARY FUNCTIONS" */
	/*         "Algorithms and implementation" */
	/*         J.M. Muller (Birkhauser) */

	/*        (*) without the additionnal raffinement to get the first coefs */
	/*         very near floating point numbers) */

	/*     minimax poly coefs */
	s2 = *s * *s;
	if (fabs(*s) <= .003032) {
		ret_val = *s * (s2 * (s2 * .40000000000000002 + .66666666666666663) + 
			2.);
	} else {
		ret_val = *s * (s2 * (s2 * (s2 * (s2 * (s2 * (s2 * (s2 * 
			.15367270224757008114 + .1525031588446936471) + 
			.18186349187499222459) + .22222138684562683797) + 
			.2857142939282938098) + .39999999996176889299) + 
			.66666666666672679472) + 2.);
	}
	return ret_val;
} /* lnp1m1_ */

