// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

//
// Check empty matrix
x = distfun_betainv ( [] , [] , [] );
assert_checkequal ( x , [] );

// Check expansion of a and b
p = [0.01 0.5 0.99];
x = distfun_betainv(p,10,5);
expected = [0.372565305114573   0.674248844713787   0.898071404278149];
assert_checkalmostequal ( x , expected , 1.e-8 );
//
// Check with expanded a and b
p = [0.01 0.5 0.99];
x = distfun_betainv(p,[8 9 10],[5 6 7]);
expected = [0.302404222173137   0.604556744392119   0.833540311685497];
assert_checkalmostequal ( x , expected , 1.e-8 );
/////////////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_betainv
//

path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","beta","betainv.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

precision = 1000*%eps * ones(20,1);
// The test at 1.e-300 has low accuracy (which is acceptable)
precision(20)=1.e-6;

nt = size(table,"r");
for k = 1 : nt
  p = table(k,1);
  a = table(k,2);
  b = table(k,3);
  expected = table(k,4);
  computed = distfun_betainv ( p , a , b );
  assert_checkalmostequal ( computed , expected , precision(k) );
  if ( %f ) then
    dX = assert_computedigits ( computed , expected );
    mprintf("Test #%3d/%3d: Digits p=%.g, X= %.1f\n",k,nt,p,dX);
  end
end

//
// Check vectorization
a = 1;
b = 3;
p = linspace(0.1,0.9,10);
x = distfun_betainv ( p , a , b );
x2 = [];
for i = 1 : 10
    x2(1,i) = distfun_betainv ( p(i) , a , b );
end
assert_checkequal(x,x2);
//
// Check upper tail
p = [0.01 0.5 0.99];
x = distfun_betainv(p,10,5,%f);
expected = [0.8980714    0.6742488    0.3725653];
assert_checkalmostequal ( x , expected , 1.e-6 );
//
// Check upper tail with R
x = distfun_betainv(0.99,10,5);
assert_checkalmostequal ( x , 0.8980714 , 1.e-6 );
x = distfun_betainv(0.99,10,5,%f);
assert_checkalmostequal ( x , 0.3725653 , 1.e-6 );
x = distfun_betainv(1.e-20,10,5,%f);
assert_checkalmostequal ( x , 0.9999781 , 1.e-6 );
