// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2011 - INRIA - Michael Baudin
// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt



// <-- ENGLISH IMPOSED -->
// <-- JVM NOT MANDATORY -->


//
// Check empty matrix
r = distfun_lognrnd ( [] , [] );
assert_checkequal ( r , [] );
//
// Set the seed so as to always get the same results.
distfun_seedset(1.);

// Test both mu and sigma expanded
c = distfun_lognrnd(1:6,(1:6)^-1);
assert_checkequal(size(c),[1 6]);
//
// // Test sigma expansion
c = distfun_lognrnd(1:6,2.0);
assert_checkequal(size(c),[1 6]);
//
// // Test mu expansion
c = distfun_lognrnd(1.0,1:6);
assert_checkequal(size(c),[1 6]);
//
// // Test with v
c = distfun_lognrnd(0,1,[3 2]);
assert_checkequal(size(c),[3 2]);
//
// // Test with m, n
c = distfun_lognrnd(0,1,3,2);
assert_checkequal(size(c),[3 2]);
//
// // Make a plot of the actual distribution of the numbers
if (%f) then
    mu = 2;
    sigma = 3;
    data = distfun_lognrnd(mu,sigma,1,1000);
    histplot(10,log(data));
    xtitle("Lognormal random variables","Log(X)","P");
    x = linspace(-10,10,1000);
    y = distfun_normpdf(x,mu,sigma);
    plot(x,y);
end

// Check mean and variance for R = distfun_exprnd ( mu )
distfun_seedset(1.);
N = 100000;
for mu = [1 3 6]
    for sigma = [1 0.1 0.01 0.001]
        computed = distfun_lognrnd(mu,sigma,[1 N]);
        // Check mean
        c = mean(computed);
        e = exp(mu+sigma^2/2);
        assert_checkalmostequal(c,e,1.e-1);
        // Check mean(log(X))
        c = mean(log(computed));
        e = mu;
        assert_checkalmostequal(c,e,1.e-1);
        // Check variance
        c = variance(computed);
        e = (exp(sigma^2)-1)*exp(2*mu+sigma^2);
        assert_checkalmostequal(c,e,1.e-1);
        // Check variance
        c = variance(log(computed));
        e = sigma^2;
        assert_checkalmostequal(c,e,1.e-1);
    end
end
//
// Check expansion of mu and sigma in R = distfun_lognrnd(mu,sigma)
//
distfun_seedset(1.);
N = 10;
mu = 1:6;
sigma = (1:6)^-1;
computed1 = [];
for i = 1:N
    computed1(i,1:6) = distfun_lognrnd(mu,sigma);
end
distfun_seedset(1.);
computed2  = [];
for i = 1:N
    for k = 1 : 6
        computed2(i,k)=distfun_lognrnd(mu(k),sigma(k));
    end
end
assert_checkequal(computed1,computed2);
