// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

//
// Check empty matrix
p = distfun_normcdf ( [] , [] , [] );
assert_checkequal ( p , [] );

//
// Test distfun_normcdf
//
computed = distfun_normcdf ( [-1 1] );
expected = [ 0.158655253931457   0.841344746068543 ];
assert_checkalmostequal ( computed , expected , 10*%eps );
//
// Test argument expansion
computed = distfun_normcdf ( [1 2 3] , 1.0 , 2.0 );
expected = [ 0.500000000000000   0.691462461274013   0.841344746068543 ];
assert_checkalmostequal ( computed , expected , 10*%eps );
//
// Test expanded arguments
computed = distfun_normcdf ( [1 2 3] , [1 1 1] , [2 2 2] );
expected = [ 0.500000000000000   0.691462461274013   0.841344746068543 ];
assert_checkalmostequal ( computed , expected , 10*%eps );

/////////////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_normcdf


path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","normcdf.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

precision = 100*%eps;

nt = size(table,"r");
for k = 1 : nt
  x = table(k,1);
  mu = table(k,2);
  sigma = table(k,3);
  expected = table(k,4);
  computed = distfun_normcdf ( x , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
  if ( %f ) then
    d = assert_computedigits ( computed , expected );
    mprintf("Test #%d/%d: Digits = %.1f\n",i,ntests,d);
  end
end

///////////////////////////////////////////////////////////////
//
// Test accuracy of distfun_normcdf

path=distfun_getpath();
dataset = fullfile(path,"tests","unit_tests","normal","normal.R.dataset.csv");
table = assert_csvread ( dataset , "," , [] , "/#(.*)/" );
table = evstr(table);

precision = 100*%eps;

ntests = size(table,"r");
for i = 1 : ntests
  x = table(i,1);
  mu = table(i,2);
  sigma = table(i,3);
  expected = table(i,5);
  computed = distfun_normcdf ( x , mu , sigma );
  assert_checkalmostequal ( computed , expected , precision );
  if ( %f ) then
    d = assert_computedigits ( computed , expected );
    mprintf("Test #%d/%d: Digits = %.1f\n",i,ntests,d);
  end
end

//
// Check vectorization
mu = 300;
sigma = 30;
x = linspace(mu-3*sigma,mu+3*sigma,10);
p = distfun_normcdf ( x , mu , sigma );
p2 = [];
for i = 1 : 10
    p2(1,i) = distfun_normcdf ( x(i) , mu , sigma );
end
assert_checkequal(p,p2);

// See upper tail (compare with R)
p = distfun_normcdf ( 7, 4, 1 );
assert_checkalmostequal ( p , 0.99865010196837 , 1.e-15 );
p = distfun_normcdf ( 7, 4, 1 , %f );
assert_checkalmostequal ( p , 0.001349898031630095 , 1.e-15 );
p = distfun_normcdf ( 15, 4, 1 , %f );
assert_checkalmostequal ( p , 1.910659574498676e-28 , 1.e-15 );
